package jst.parsetree;

import java.io.*;
import jst.symtab.*;
import java.util.*;
import parser.parsetree.*;
import parser.parsetree.visitor.*;
import org.xml.sax.InputSource;

/** A parse tree is described by a root Nonterminal and its children.
 *  It is loaded from an XML source file, which was earlier produced by a parser.
 */
public class ParseTree {

	/** The root node of the parse tree. */
	private JstParseTreeBranch root;

	/** The name of the file from which this parse tree was loaded. */
    private String fileName;

	/** Construct a parse tree by loading it from a source file.
     * @param source The file to be loaded.
     */
	public ParseTree(File source) {
    	fileName = source.getName(); // Remember where the tree came from.
        FileReader file;
        try {
			file = new FileReader(source);
        }
        catch (FileNotFoundException e) {
        	throw new RuntimeException(e.getMessage());
        }

        InputSource in = new InputSource(file);
        XMLTreeLoader loader = new XMLTreeLoader();
        List messages = new ArrayList();
		root = loader.load(in, messages);

        if (root == null) { // Fix this
            System.out.println("Couldn't load parse tree: " + messages);
            throw new RuntimeException(messages.toString());
        }


		/*
		PrintParseTreeVisitor v= new PrintParseTreeVisitor();
		root.accept(v);
		*/
	}
    

	/** Get the name of file that supplied this parse tree. */
	public String getFileName() {
    	return fileName;
    }

	/** Get the root node of the parse tree. */
	public JstParseTreeBranch getRoot() {
    	return root;
    }

	/** Save the parse tree as XML.
     *  @param out The destination of prints.
     */
	public void save(PrintWriter out, String indent) {
		out.println(indent + "<parseTree file='" + fileName + "'>");
        //root.save(out, indent + " ");
		out.println(indent + "</parseTree>");
    }
}


