
package jst.symtab;

import java.util.*;

import jst.parsetree.*;
import jst.symtab.visitor.ModelVisitor;

public abstract class Decl {
	
    /**
     * @supplierCardinality 1 
     * @supplierRole ownerScope
     *
	*/
	protected Decl ownerDecl;  // Null for default package only.

	protected String simpleName; // Never null or empty. Anonymous decls get an artificial name.

	// May be null if source unknown
	protected JstParseTreeBranch source;
	
	// @pre theSimpleName != null && theSimpleName != "" 
	protected Decl(Decl theOwner, String theSimpleName, JstParseTreeBranch theSource) {
		ownerDecl = theOwner;

    	if (theSimpleName == null || theSimpleName.length() == 0)
        	simpleName = getOwnerScope().generateName();
		else
			simpleName = theSimpleName;

        source = theSource;

		init();
    }

	protected Decl(Decl theOwner, Decl that, Map<TypeParameter, ActualType> swaps) {
		this(theOwner, that.getSimpleName() + "_" + swaps.values(), that.getSource());
	}

	protected void init() {
	}

	// Convert a Scope into a Decl.
	public Decl asDecl() {
		return this;
	}

	public Scope asScope() {
		return null;
	}

	public Decl getOwnerDecl() {
		return ownerDecl;
	}

	public Scope getOwnerScope() {
    	return ownerDecl.getNearestScope();
    }

	public Scope getNearestScope() {

		if (asScope() != null)
			return asScope();

    	return ownerDecl.getNearestScope();
	}

	public PackageDecl getOwnerPackage() {
		return getOwnerScope().getOwnerPackage();
    }

	public UserType getOwnerUserType() {
		if(ownerDecl == null) {
			return null;
		}
		return ownerDecl.getNearestUserType();
	}

	public UserType getNearestUserType() {
		if(ownerDecl == null) {
			return null;
		}
		return ownerDecl.getNearestUserType();
	}

	public SourceFile getOwnerSourceFile() {
		return getOwnerDecl().getOwnerSourceFile();
	}

	public String getOwnerFileName() {
		Decl src = getOwnerSourceFile();
		if (src == null)
			src = getOwnerUserType();

		return src.getCanonicalName();
	}

	public String getSimpleName() {
    	return simpleName;
    }

    // This must never return an empty string, because it is used for ids in XML.
	public abstract String getCanonicalName();

	public JstParseTreeBranch getSource() {
    	return source;
    }

	// Not all Decls are members, but for simplicity, we treat them as if they were.
    // In other words, any decl can have modifiers (static, private, final, etc),
    // and can be tested for accessibility by a class.
    // For Decls that aren't really members, they default to unmodified and public.

	// Must be overridden by member decls (UserType, FieldDecl, OperationDecl.
	public int getModifier() {
    	return 0;
    }

    public void setModifier(int theModifier) {
    	throw new RuntimeException("Tried to setModifier() of non-member");
    }

    public boolean accessible(ReferenceType looker) {
    	return true;
    }

    public abstract void accept(ModelVisitor visitor);

	public String toString() {
    	// return getCanonicalName();
    	return getSimpleName();
    }
}
