package jst.symtab;

import java.lang.reflect.Modifier;
import java.util.*;

import jst.symtab.visitor.ModelVisitor;
import jst.parsetree.*;
import jst.symtab.link.*;

public class FieldInitializer extends ExecutableCode {

	// Eww, can make an anonymous inner class in a field initializer.
	// Who should own it? 
	// The FieldInitializer owns it, but is not a scope.
	// This is OK because the inner class can never be looked up.
	protected Set<UserType> anonInnerClasses;

    /**
     * Construct a field initializer.
     */
	public FieldInitializer(FieldDecl ownerField, JstParseTreeBranch theSource) {
        super(ownerField, ownerField.getSimpleName() + "@INIT", theSource);

        ownerField.setInitializer(this);
		anonInnerClasses = new HashSet<UserType>();
    }

	public FieldInitializer(FieldDecl newOwnerField, FieldInitializer that, Map<TypeParameter, ActualType> swaps) {
		super(newOwnerField, that, swaps);

		for (Link thatLink : references)
			references.add(thatLink.copy(this));

        newOwnerField.setInitializer(this);
	}

	public void copy(FieldDecl newOwnerField, Map<TypeParameter, ActualType> swaps) {
		new FieldInitializer(newOwnerField, this, swaps);
	}

    public String getCanonicalName() {
       return "FLI_" + getOwnerScope().getCanonicalName().substring(4) + "." + simpleName;
    }

    // Used for telling if the context is static.
    public int getModifier() {
        return getOwnerDecl().getModifier() & Modifier.STATIC;
    }

	// FieldInitializers aren't scopes, but can contain anonymous inner classes.
	public void addType(UserType anonInnerClass) {
		anonInnerClasses.add(anonInnerClass);
	}

	public Set<UserType> getAnonInnerClasses() {
		return anonInnerClasses;
	}

    public void accept(ModelVisitor visitor) {
        visitor.visitFieldInitializer(this);
    }
}
