
package jst.symtab;

import java.util.*;
import jst.symtab.visitor.ModelVisitor;

public class ParameterizedClassType extends ClassType {

	protected GenericClassType rawType;

	protected Map<String, ActualType> typeArguments;

	// Owner scope differs from actual owner (the rawType)!
	protected ParameterizedClassType(GenericClassType theRawType, Map<TypeParameter, ActualType> swaps) {
		super(theRawType, theRawType, swaps);

		rawType = theRawType;

		typeArguments = new LinkedHashMap<String, ActualType>(swaps.size());

		for (Map.Entry<TypeParameter, ActualType> entry : swaps.entrySet())
			typeArguments.put(entry.getKey().getSimpleName(), entry.getValue());

		rawType.addParameterizedClass(this);
	}

	public void copy(SourceFile newOwnerSourceFile, Map<TypeParameter, ActualType> swaps) {
		assert false;
	}

	public void copy(PackageDecl newOwnerPackage, Map<TypeParameter, ActualType> swaps) {
		assert false;
	}

	public void copy(UserType newOwnerUserType, Map<TypeParameter, ActualType> swaps) {
		assert false;
	}

	public void copy(Block newOwnerBlock, Map<TypeParameter, ActualType> swaps) {
		assert false;
	}

	public Scope getOwnerScope() {
		return getOwnerDecl().getOwnerScope();
	}

	public List<ActualType> getActualTypes() {
		List<ActualType> actualTypes = new ArrayList<ActualType>(typeArguments.size());

		// Preserve order.
		for (Map.Entry<String, ActualType> entry : typeArguments.entrySet())
			actualTypes.add( entry.getValue());

		return actualTypes;
	}

	public TypeDecl lookupType(boolean widen, String typeName, ReferenceType looker) {
		ActualType arg = typeArguments.get(typeName);
		if (arg != null)
			return arg;

		return super.lookupType(widen, typeName, looker);
	}

    public void accept(ModelVisitor visitor) {
        visitor.visitParameterizedClassType(this);
    }
}



