
package jst.symtab;
import java.util.*;

/**
 * Scopes form a containment hierarchy.  It is essentially lexical, except that it also
 * includes packages (which aren't lexical).  The root scope is the default package.
 *
 * Anything that can contain declarations is a scope: Packages, source files, reference
 * types, operations (they contain parameters) and blocks.
 *
 * The owner scope of UserType scope may be a package scope or a sourcefile scope.
 * If source is available, the owner will be the source file, which in turn is owned by the package.
 * If source is unavailable, the owner will be the package.  This reflects lookup behaviour,
 * but see comments in lookup methods of symbol table for trickyness with type-import-on-demands.
 */
public interface Scope {

	public Decl asDecl();
	public Scope getOwnerScope();
	public PackageDecl getOwnerPackage();
	public UserType getOwnerUserType();

	public String getCanonicalName();
	public String generateName(); // Generate artificial name for decl in this scope.

	/*
	public void addType(TypeDecl type);
	public void addMethod(MethodDecl type);
	public void addVariable(VariableDecl type);
	*/

	// public boolean addType(UserType type); // Return true if this scope can contain the type.
	public TypeDecl lookupType(boolean widen, String typeName, ReferenceType looker);
	public MethodDecl lookupMethod(boolean widen, String methodName, List params, ReferenceType looker);
	public VariableDecl lookupVariable(boolean widen, String variableName, ReferenceType looker);
}
