package jst.symtab.link;

import parser.parsetree.SyntaxTreeLeaf;
import jst.app.JSTMessage;
import jst.symtab.*;

public class WildcardLink implements ActualTypeLink {

	protected Identifier token;

	protected ReferenceTypeLink bound;

	protected boolean isUpperBound;

	protected WildcardType wildcardType;

	public WildcardLink(Identifier theToken) {
		token = theToken;
	}

	public WildcardLink(Decl newOwner, WildcardLink that) {
		token = that.token;
		bound = that.bound.copyType(newOwner);
		isUpperBound = that.isUpperBound;
	}
	
	public SyntaxTreeLeaf getSource(){
		return token.getSource();
	}

	public WildcardLink copy(Decl newOwner) {
		return new WildcardLink(newOwner, this);
	}

	public WildcardLink copyType(Decl newOwner) {
		return copy(newOwner);
	}

	public void setBound(boolean upper, ReferenceTypeLink theBound) {
		isUpperBound = upper;
		bound = theBound;
	}

	public String getName() {
		return token.getName();
	}

	public int getLine() {
		return token.getLine();
	}

	public int getColumn() {
		return token.getColumn();
	}

	public String getOwnerFileName() {
		return "dunno";
	}

	public String getFullName() {
		return token + (isUpperBound? " extends " : " super " + bound.getFullName());
	}

	public boolean resolve() {
		wildcardType = new WildcardType(null, token.toString(), null);
		// No ownerScope.  Dunno who should own it!

		if (bound != null) {
			if (!bound.resolve())
				return false;

			wildcardType.setBound(isUpperBound, bound.getReferenceType());
		}

		return true;
	}

	public JSTMessage getMessage() {
		if (bound != null && bound.getDecl() == null)
			return bound.getMessage();

		return null;
	}

	public Decl getDecl() {
		return getType();
	}

	public TypeDecl getType() {
		return getActualType();
	}

	public ActualType getActualType() {
		return getWildcardType();
	}

	public WildcardType getWildcardType() {
		return wildcardType;
	}

	public String toString() {
		return getOwnerFileName() + ":" + getFullName();
	}
}
