package parser.pda;
import parser.pda.tokenStream.Looker;
import parser.parsetree.*;
import parser.parsetree.treeFactory.SyntaxTreeFactory;
import parser.pda.stateMachine.State;
import parser.grammar.Production;
import java.util.*;
import parser.pda.stateMachine.StateMachine;

public class DeterministicParser extends GraphDrivenParser {

    /**
     * @supplierCardinality 1
     * @supplierRole top 
     */
    protected LinearStackNode top;

    public DeterministicParser(StateMachine machine, SyntaxTreeFactory theTreeBuilder) {
        super(machine, theTreeBuilder);
    }

    protected void start() {
        makeStartNode(stateMachine.getStartState());
    }

    protected void makeStartNode(State state) {
        top = new LinearStackNode(this, state);
    }

    protected boolean step(Looker looker) {
        LinearStackNode oldTop = top;

        top.act(looker);

        return top != oldTop && top.getState().getSymbol().isTerminal();
    }

    protected boolean accept() {
        return top.getState() == stateMachine.getStopState();
    }

    public void shift(Token input, State dst) {
        top = top.makeShiftNode(dst, input);
    }

    public void goTo(LinearStackNode prev, State dst, Production prod, List kids, Looker input) {
        SyntaxTreeBranch tree = treeBuilder.makeBranch(prod, kids);

        top = prev.makeGotoNode(dst, tree);
        top.act(input);
    }

    protected SyntaxTreeBranch getResult() {
        if (!accept())
            return null;

        return top.getResult();
    }

    protected Set getExpected() {
        return top.getExpected();
    }

    protected String getContext() {
        return top.getContext();
    }
}
