package parser.pda;

import parser.parsetree.*;
import parser.grammar.*;
import parser.pda.tokenStream.*;
import parser.parsetree.treeFactory.*;

import util.*;
import java.util.*;

public abstract class Parser {

    protected SyntaxTreeFactory treeBuilder;

    protected MessagePrinter printer;

    public Parser(SyntaxTreeFactory theTreeBuilder) {
        treeBuilder = theTreeBuilder;
        setPrinter(new MessagePrinter());
    }

    public void setPrinter(MessagePrinter thePrinter) {
        printer = thePrinter;
    }

    public SyntaxTreeBranch parse(TokenStream tokenStream) {
        start ();
        boolean ok = true;

        while (ok && !accept()) {
            tokenStream.advance();

            if (!step(tokenStream)) {
                error(tokenStream);
                ok = recover(tokenStream);
            }
        }

        return ok? getResult() : null;
    }

    protected abstract void start();

    protected abstract boolean step(Looker looker);

    protected abstract boolean accept();

    protected abstract SyntaxTreeBranch getResult();

    protected void error(Looker looker) {
        Token token = looker.first();
        Symbol tokenType = token.getSymbolType();

        if (tokenType == null)
            printer.print(new Message(ParserMessage.LEXICAL_ERROR, token.toString()));
        else
            printer.print(new Message(ParserMessage.SYNTAX_ERROR, token.toString()));

		String expected = "";

		Iterator lookIter = getExpected().iterator();
		while (lookIter.hasNext()) {
			List look = (List) lookIter.next();

			Iterator terminalIter = look.iterator();
			while (terminalIter.hasNext()) {
				Terminal terminal = (Terminal) terminalIter.next();

				expected += terminal;

				if (terminalIter.hasNext())
					expected += " ";
			}

			if (lookIter.hasNext())
				expected += ", ";
		}

        printer.print(new Message(ParserMessage.EXPECTED_TOKENS, expected));
        printer.print(new Message(ParserMessage.PARSER_CONTEXT, getContext()));
    }

    protected boolean recover(TokenStream tokenStream) {
        return false;
    }

    protected abstract Set getExpected();

	protected abstract String getContext();
}
