package parser.pda;
import java.util.*;
import parser.pda.stateMachine.State;
import parser.parsetree.SyntaxTree;
import parser.grammar.Production;
import parser.parsetree.SyntaxTreeBranch;
import parser.pda.tokenStream.Looker;

public class ShiftNode extends GSSNode {

    /** @associates parser.pda.GSSNode
     * @supplierRole previous
     * @supplierCardinality 0..**/
    protected Map previous; // GSSNode ==> SyntaxTree

    // Just for start state
    public ShiftNode(NondeterministicParser theParser, State theState) {
        super(theParser, theState);
        previous = new HashMap();
    }

    public ShiftNode(NondeterministicParser theParser, GSSNode prev, State theState, SyntaxTree tree) {
        this(theParser, theState);
        addPrevious(prev, tree);
    }

    public void addPrevious(GSSNode prev, SyntaxTree tree) {
        previous.put(prev, tree);
    }

    protected void reducePrevious(int n, Production prod, List kids, Looker input) {
        Iterator prevIter = previous.entrySet().iterator();
	    while (prevIter.hasNext()) {
	        Map.Entry entry = (Map.Entry) prevIter.next();
	        GSSNode prev = (GSSNode) entry.getKey();
	        SyntaxTree tree = (SyntaxTree) entry.getValue();

            kids.add(0, tree);
	        prev.reduce(n, prod, kids, input);
            kids.remove(0);
	    }
    }

    public SyntaxTree getTree(GSSNode prev) {
        return (SyntaxTree) previous.get(prev);
    }

    public SyntaxTreeBranch getResult() {
        if (previous.size() != 1)
            return null; // Shouldn't be possible.

        GSSNode prev = (GSSNode) previous.keySet().iterator().next();
        return prev.getResult();
    }
}
