package parser.pda;

import parser.parsetree.*;
import util.*;
import parser.parsetree.treeFactory.*;
import parser.grammar.*;

import java.util.*;
import parser.pda.tokenStream.Looker;
import parser.pda.stateMachine.DeterministicStateTable;
import parser.pda.stateMachine.StateTable;

public abstract class SimpleParser extends TableDrivenParser {

    protected DeterministicStateTable stateTable;

    /**
     * @supplierCardinality 0..1
     * @supplierRole stack 
     */
	protected SimpleStackNode stack;

    public SimpleParser(DeterministicStateTable theStateTable, SyntaxTreeFactory theBuilder) {
	    super(theBuilder);
        stateTable = theStateTable;
	}

	// Called at the start of each parse.
	// must call this after startstate is set up -- i.e. when subclass constructor is done.
	protected void start() {
		stack = new SimpleStackNode(stateTable.getStartState());
	}

	protected boolean step(Looker looker) {
        Token token = looker.first();
		Symbol tokenType = token.getSymbolType();
		if (tokenType == null)
			return false;

        Production reduce = stateTable.getReduction(stack.getState(), tokenType);

		while (reduce != null) {
            List kids  = new LinkedList(); // SyntaxTreeNodes
                                           // (Inefficient, but tidy.)                                         

			for (int i = reduce.getSize(); i > 0; i--) {
				kids.add(0, stack.getTree());
				stack = stack.pop();
			}

			SyntaxTree branch = treeBuilder.makeBranch(reduce, kids);
            Symbol gotoSymbol = reduce.getLhs();
            int gotoState = stateTable.getTransition(stack.getState(), gotoSymbol);
			stack = stack.push(gotoState, branch);
            reduce = stateTable.getReduction(stack.getState(), tokenType);
		}

        int shiftState = stateTable.getTransition(stack.getState(), tokenType);
	    if (shiftState > 0) {
			stack = stack.push(shiftState, token);
		 	return true;
	    }

		return false;
	}

    protected boolean accept() {
	    return stack.getState() == stateTable.getStopState();
	}

    protected SyntaxTreeBranch getResult() {
        stack = stack.pop(); // Pop eof symbol.
        SyntaxTree tree = stack.getTree();
        return tree.asBranch();
    }

    protected Set getExpected() {
        return stateTable.getExpected(stack.getState());
	}

	protected String getContext() {
        return stateTable.getContext(stack.getState());
	}

}
