package parser.pda;

import parser.parsetree.*;
import parser.pda.stateMachine.*;
import parser.pda.tokenStream.*;
import parser.grammar.*;

import java.util.*;
import util.MultiMap;

public abstract class StateStackNode implements  PdaActor {

    protected State state;

    protected StateStackNode(State theState) {
        state = theState;
    }

    public void act(Looker input) {
        state.act(input, this);
    }

    public void reduce(Production prod, Looker input) {
        reduce(prod.getSize(), prod, new LinkedList(), input);
	}

    public void reduce(int n, Production prod, List kids, Looker input) {

        if (n == 0) {
            State gotoDst = state.getTransition(prod.getLhs());
            assert gotoDst != null : state + " " + prod;
            goTo(gotoDst, prod, kids, input);
        }
	    else
            reducePrevious(n-1, prod, kids, input);
	}

    protected abstract void reducePrevious(int n, Production prod, List kids, Looker input);

    protected abstract void goTo(State dst, Production prod, List kids, Looker input);

    public Set getExpected() {
        return state.getExpected();
    }

    public State getState() {
        return state;
    }

	public String getContext() {
		return state.getContext();
	}

    public abstract SyntaxTreeBranch getResult();
}
